# Name :          All Faces Coloured
# Description :   Colours all faces with a random selection of the named
#   			        Sketchup colours
# Author[s] :     CJ
#
# TODO:           Ensure random colours are different.
#                 Fix so that components definitions are included.
#                 Fix undo stack (single command)
#                 Future: Options for groups & components to be single colour
#                 Future: 'Reset' button to set all materials to default
#
# 1.0  (20.01.2015) - Version 1
#-------------------------------------------------------
require 'sketchup.rb'
#-------------------------------------------------------

module CJ_ColourFaces

  ### Add to Plugins menu on startup

  unless file_loaded?( __FILE__ )
    menu = UI.menu( 'Plugins' )
    menu.add_item( 'Colour Faces' ) { self.colourfaces(@entities) }
  end

  ### Set all variables

  @colour_selection = Sketchup::Color.names
  @colour_number = @colour_selection.length
  @colour_current = @colour_selection[rand(@colour_number)]
  @model = Sketchup.active_model
  @entities = Sketchup.active_model.entities

  ### Main method to recursively colour all faces

  def self.colourfaces(entities)
    @model.start_operation("Colour faces", true, true, false)

    entities.each do |entity|
      if entity.is_a? Sketchup::Face
        if entity.material == nil
          entity.material = @colour_current
          self.softcheck(entity)
          @colour_current = @colour_selection[rand(@colour_number)]
        else next
        end
      elsif entity.is_a? Sketchup::Group || Sketchup::ComponentDefinition
        entities = entity.entities
        self.colourfaces(entities)
      end
    end
    
    @model.commit_operation
  end # self.colourfaces

  ### Method to check for adjoined softened faces & colour them

  def self.softcheck(face)
    face.edges.each do |edge|
      if edge.soft?
      @face_array = edge.faces 
      @face_array.delete(face)
        if @face_array[0].material == nil
          @face_array[0].material = @colour_current
          self.softcheck(@face_array[0])
        end
      else next
      end
    end
  end

end # CJ_ColourFaces